# src/mpc.py
"""
Módulo para lidar com o protocolo MCP (Model Context Protocol).
"""
import json
import logging
import os
import sys
import uuid
from typing import Optional, Dict, Any

from src.app import AgentOrchestrator
from src.core.logger import trace, agent_span, generation_span
from src.core.models import ModelManager
from src.core.db import DatabaseManager

from openai import OpenAI

# Configuração básica de logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[
        logging.StreamHandler(sys.stdout),
        logging.FileHandler("logs/mcp_server.log")
    ]
)

logger = logging.getLogger(__name__)

try:
    from mcp_sdk import BaseMCPHandler, Message, Response
except ImportError:
    # Mock classes para testes
    class Message:
        """Mock da classe Message do SDK MCP."""
        def __init__(self, content: str, metadata: Dict[str, Any]):
            self.content = content
            self.metadata = metadata

    class Response:
        """Mock da classe Response do SDK MCP."""
        def __init__(self, content: Dict[str, Any], metadata: Optional[Dict[str, Any]] = None):
            self.content = content
            self.metadata = metadata or {}

    class BaseMCPHandler:
        """Mock da classe BaseMCPHandler do SDK MCP."""
        def __init__(self):
            self.initialized = False
            self.hook = "/prompt-tdd"

        def initialize(self, api_key: str) -> None:
            """Inicializa o handler com a chave da API."""
            self.initialized = True

        def run(self) -> None:
            """Executa o loop principal do handler."""
            while True:
                try:
                    line = sys.stdin.readline()
                    if not line:
                        break

                    data = json.loads(line)
                    message = Message(data["content"], data["metadata"])
                    response = self.handle_message(message)
                    print(json.dumps({
                        "content": response.content,
                        "metadata": response.metadata
                    }))
                except Exception as e:
                    logging.error(f"Erro ao processar mensagem: {str(e)}")
                    break

        def handle_message(self, message: Message) -> Response:
            """Processa uma mensagem e retorna uma resposta."""
            raise NotImplementedError()


class PromptManager:
    def __init__(self):
        self.templates: Dict[str, str] = {}
        logger.info("PromptManager inicializado")
        
    def add_template(self, name: str, template: str) -> None:
        """Adiciona um novo template de prompt."""
        self.templates[name] = template
        logger.info(f"Template '{name}' adicionado")
        
    def get_template(self, name: str) -> Optional[str]:
        """Recupera um template de prompt pelo nome."""
        template = self.templates.get(name)
        if not template:
            logger.warning(f"Template '{name}' não encontrado")
        return template
        
    def format_prompt(self, template_name: str, **kwargs) -> Optional[str]:
        """Formata um prompt usando um template e variáveis."""
        template = self.get_template(template_name)
        if not template:
            return None
            
        try:
            return template.format(**kwargs)
        except KeyError as e:
            logger.error(f"Erro ao formatar prompt: variável {e} não fornecida")
            return None
        except Exception as e:
            logger.error(f"Erro ao formatar prompt: {str(e)}")
            return None 

class LLMProvider:
    def __init__(self):
        self.api_key = os.getenv("OPENAI_API_KEY")
        if not self.api_key:
            logger.warning("OPENAI_API_KEY não encontrada nas variáveis de ambiente")
        self.client = OpenAI(api_key=self.api_key)
        
    def generate(self, prompt: str, options: Dict[str, Any]) -> Optional[str]:
        try:
            model = options.get("model", "gpt-3.5-turbo")
            temperature = options.get("temperature", 0.7)
            format = options.get("format", "json")
            
            logger.info(f"Gerando resposta com modelo {model} (temperatura: {temperature})")
            
            # Ajusta o prompt para gerar resposta no formato correto
            if format == "markdown":
                prompt = f"Por favor, formate sua resposta em Markdown com:\n- Títulos e subtítulos\n- Listas\n- Destaques\n- Tabelas quando relevante\n\nPrompt: {prompt}"
            
            response = self.client.chat.completions.create(
                model=model,
                messages=[{"role": "user", "content": prompt}],
                temperature=temperature
            )
            
            content = response.choices[0].message.content
            
            # Retorna sempre um dicionário estruturado
            return {
                "content": content,
                "metadata": {
                    "status": "success",
                    "format": format
                }
            }
                
        except Exception as e:
            logger.error(f"Erro ao gerar resposta: {str(e)}")
            return None

class MCPHandler:
    """Manipulador do protocolo MCP."""

    def __init__(self, llm_provider: 'LLMProvider', prompt_manager: 'PromptManager'):
        """Inicializa o manipulador MCP."""
        self.orchestrator = None
        self.models = ModelManager()
        self.llm_provider = llm_provider
        self.prompt_manager = prompt_manager
        self.db = DatabaseManager()
        self.session_id = str(uuid.uuid4())
        logger.info("MCPHandler inicializado com sucesso")

    def initialize(self, api_key: Optional[str] = None):
        """Inicializa o orquestrador com a chave da API."""
        logger.info("Inicializando MCPHandler...")
        self.orchestrator = AgentOrchestrator(api_key=api_key)
        self.models.configure(
            model="gpt-3.5-turbo",
            temperature=0.7
        )
        logger.info("MCPHandler inicializado com sucesso")

    @trace(workflow_name="MCP Workflow")
    @agent_span()
    def process_message(self, message: Dict[str, Any]) -> Optional[str]:
        """
        Processa uma mensagem recebida.
        
        Args:
            message: Mensagem a ser processada
            
        Returns:
            Resposta processada ou None em caso de erro
        """
        try:
            content = message.get("content")
            metadata = message.get("metadata", {})
            
            if not content:
                logger.warning("Mensagem recebida sem conteúdo")
                return None
                
            logger.info(f"Processando mensagem: {content}")
            logger.info(f"Metadata: {metadata}")
            
            # Registra a execução no banco
            run_id = self.db.log_run(
                session_id=self.session_id,
                input_text=content,
                last_agent=None,  # Será atualizado após a execução
                output_type=metadata.get("options", {}).get("format", "json")
            )
            
            with generation_span(name="LLM Generation"):
                response = self.llm_provider.generate(content, metadata.get("options", {}))
            
            if response:
                # Atualiza o registro com a resposta
                self.db.log_raw_response(run_id, response)
                
                # Registra a resposta final
                self.db.log_run_item(
                    run_id=run_id,
                    item_type="MessageOutput",
                    raw_item={"content": response["content"]}
                )
                
                logger.info(f"Resposta gerada: {response['content']}")
                return response["content"]
            
            return None
            
        except Exception as e:
            logger.error(f"Erro ao processar mensagem: {str(e)}")
            return None

    def handle_message(self, message: Message) -> Response:
        """
        Manipula uma mensagem MCP.
        
        Args:
            message: Mensagem MCP recebida
            
        Returns:
            Resposta MCP formatada
        """
        try:
            # Processa a mensagem
            result = self.process_message({
                "content": message.content,
                "metadata": message.metadata
            })
            
            if result:
                # Busca o último registro do banco para metadados
                history = self.db.get_run_history(limit=1)
                metadata = {}
                
                if history:
                    last_run = history[0]
                    metadata = {
                        "last_agent": last_run["last_agent"],
                        "output_type": last_run["output_type"],
                        "session_id": last_run["session_id"],
                        "items_count": len(last_run["items"]),
                        "guardrails_count": len(last_run["guardrails"]),
                        "raw_responses_count": len(last_run["raw_responses"])
                    }
                
                return Response({"content": result}, metadata=metadata)
            
            return Response({"error": "Falha ao processar mensagem"}, metadata={"status": "error"})
            
        except Exception as e:
            logger.error(f"Erro ao manipular mensagem: {str(e)}")
            return Response({"error": str(e)}, metadata={"status": "error"})

    def run(self):
        """Executa o manipulador MCP."""
        try:
            # Lê o arquivo de pipe
            pipe_file = "logs/mcp_pipe.log"
            logger.info(f"Iniciando leitura do arquivo: {pipe_file}")
            
            with open(pipe_file, "r") as f:
                content = f.read().strip()
                
            if not content:
                logger.warning("Arquivo vazio")
                return
                
            # Processa o conteúdo
            try:
                message_data = json.loads(content)
                message = Message(
                    content=message_data["content"],
                    metadata=message_data.get("metadata", {})
                )
            except json.JSONDecodeError:
                message = Message(content=content, metadata={})
            
            # Processa a mensagem
            self.handle_message(message)
            
            # Remove o arquivo após processamento
            os.remove(pipe_file)
            logger.info("Arquivo removido")
            
        except Exception as e:
            logger.error(f"Erro ao executar MCP: {str(e)}")
            raise

if __name__ == "__main__":
    # Executar como serviço standalone
    llm_provider = LLMProvider()
    prompt_manager = PromptManager()
    handler = MCPHandler(llm_provider, prompt_manager)
    handler.initialize(api_key=None)  # A chave será obtida das variáveis de ambiente
    handler.run()



"""
Utilitários para o sistema.
Módulo com funções comuns usadas em diferentes partes do sistema.
"""
import os
import re
import json
import yaml
from datetime import datetime
from pathlib import Path
from typing import Optional, Any, List, Dict, Tuple, Union

from src.core.logger import get_logger, log_execution

logger = get_logger(__name__)

@log_execution
def setup_paths():
    """Configura e valida os caminhos base do projeto"""
    logger.info("INÍCIO - setup_paths | Configurando caminhos base")
    
    try:
        # Constrói caminhos dentro do projeto
        base_dir = Path(__file__).resolve().parent.parent.parent
        logger.debug(f"Diretório base: {base_dir}")

        # Configurações de caminhos
        log_dir = os.path.join(base_dir, 'run', 'logs')
        configs_dir = os.path.join(base_dir, 'configs')

        # Criar diretórios se não existirem
        os.makedirs(log_dir, exist_ok=True)
        os.makedirs(configs_dir, exist_ok=True)
        
        logger.info("SUCESSO - Caminhos configurados e validados")
        return base_dir, log_dir, configs_dir
        
    except Exception as e:
        logger.error(f"FALHA - setup_paths | Erro: {str(e)}", exc_info=True)
        raise

# Executa a configuração de caminhos
BASE_DIR, LOG_DIR, CONFIGS_DIR = setup_paths()

def load_config(config_file: str) -> dict:
    """
    Carrega configurações de um arquivo YAML.
    
    Args:
        config_file: Caminho do arquivo de configuração
        
    Returns:
        dict: Configurações carregadas
    """
    try:
        with open(config_file, 'r', encoding='utf-8') as f:
            return yaml.safe_load(f)
    except Exception as e:
        logger.error(f"FALHA - Erro ao carregar configuração: {str(e)}", exc_info=True)
        raise

# Carrega configurações
CONFIG = load_config('src/configs/kernel.yaml')

def get_env_var(name: str, default: Optional[str] = None, args_value: Optional[str] = None) -> Optional[str]:
    """
    Obtém uma variável de ambiente, priorizando args sobre os.environ.

    Args:
        name: Nome da variável.
        default: Valor padrão se a variável não existir.
        args_value: Valor passado via argumentos de linha de comando.

    Returns:
        Valor da variável ou None se não encontrada.
    """
    # Prioriza valor passado via args
    if args_value is not None:
        return args_value
    
    # Se não houver valor em args, tenta os.environ
    return os.environ.get(name, default)


def get_env_status(context: str = "cli") -> Dict[str, Dict[str, bool]]:
    """
    Verifica o status das variáveis de ambiente necessárias.

    Args:
        context: Contexto de execução ("cli", "github", "all")

    Returns:
        Dicionário com o status de cada variável.
    """
    # Obtém variáveis das configurações
    required_vars = CONFIG["required_vars"]
    optional_vars = CONFIG["optional_vars"]

    # Seleciona as variáveis baseado no contexto
    if context == "all":
        selected_required = {k: v for d in required_vars.values() for k, v in d.items()}
        selected_optional = {k: v for d in optional_vars.values() for k, v in d.items()}
    else:
        selected_required = required_vars.get(context, {})
        selected_optional = optional_vars.get(context, {})

    # Verifica variáveis obrigatórias
    for var in selected_required:
        selected_required[var] = bool(get_env_var(var))

    # Verifica variáveis opcionais
    for var in selected_optional:
        selected_optional[var] = bool(get_env_var(var))

    return {
        "required": selected_required,
        "optional": selected_optional,
        "all_required_set": all(selected_required.values()),
    }


def validate_env(component: str) -> None:
    """
    Valida variáveis de ambiente necessárias.
    
    Args:
        component: Componente que está solicitando a validação
    """
    logger = get_logger(__name__)
    
    # Carrega configurações
    config = load_config('src/configs/kernel.yaml')
    
    # Obtém variáveis requeridas para o componente
    required_vars = config['required_vars'].get(component, [])
    
    # Se estiver publicando, adiciona PYPI_TOKEN como requerido
    if os.environ.get('PUBLISHING') == 'true':
        required_vars.append('PYPI_TOKEN')
    
    # Verifica variáveis
    missing_vars = []
    for var in required_vars:
        if not get_env_var(var):
            missing_vars.append(var)
            
    if missing_vars:
        raise ValueError(
            f"Variáveis de ambiente obrigatórias não definidas: {', '.join(missing_vars)}\n"
            "IMPORTANTE: Não use arquivos .env. Configure as variáveis diretamente no ambiente ou via argumentos."
        )

def mask_sensitive_data(data: Union[str, Dict[str, Any], List[Any]]) -> Union[str, Dict[str, Any], List[Any]]:
    """
    Mascara dados sensíveis em strings, dicionários ou listas.

    Args:
        data: Dados a serem mascarados. Pode ser uma string, dicionário ou lista.

    Returns:
        Dados com informações sensíveis mascaradas.
    """
    if isinstance(data, str):
        # Mascara tokens e chaves de API usando padrões do config
        patterns = CONFIG["sensitive_data_patterns"]
        masked = data
        for pattern_config in patterns:
            masked = re.sub(pattern_config["pattern"], pattern_config["replacement"], masked)
        return masked
    elif isinstance(data, dict):
        return {k: mask_sensitive_data(v) for k, v in data.items()}
    elif isinstance(data, list):
        return [mask_sensitive_data(item) for item in data]
    return data

def mask_partially(text, mask_str='***'):
    """
    Mascara parcialmente conteúdo sensível, mantendo caracteres iniciais.
    
    Args:
        text: Texto a ser mascarado
        mask_str: String de substituição
        
    Returns:
        Texto mascarado
    """
    if not text or len(text) < 8:
        return mask_str
        
    # Mostra os primeiros 4 caracteres e mascara o resto
    visible = min(4, len(text) // 3)
    return text[:visible] + mask_str

def format_timestamp(timestamp: str) -> str:
    """
    Formata timestamp para exibição.
    
    Args:
        timestamp: String com o timestamp
        
    Returns:
        String formatada no padrão YYYY-MM-DD HH:MM:SS
    """
    try:
        dt = datetime.fromisoformat(timestamp)
        return dt.strftime("%Y-%m-%d %H:%M:%S")
    except:
        return timestamp

def format_json(data: str) -> str:
    """
    Formata JSON para exibição.
    
    Args:
        data: String com o JSON
        
    Returns:
        String formatada com indentação
    """
    try:
        parsed = json.loads(data)
        return json.dumps(parsed, indent=2)
    except:
        return data

"""
Utilitário para validação de tokens e chaves de API.
"""

class TokenValidator:
    """
    Validador de tokens de API e chaves de ambiente.
    Garante que os tokens necessários estejam presentes e válidos.
    """
    
    @staticmethod
    def validate_token(token: Optional[str], token_name: str, required: bool = True) -> bool:
        """
        Valida se um token está presente e tem uma estrutura mínima válida.
        
        Args:
            token: O token a ser validado
            token_name: Nome do token para mensagens de erro
            required: Se o token é obrigatório
            
        Returns:
            bool: True se o token for válido, False caso contrário
            
        Raises:
            ValueError: Se o token for obrigatório e estiver ausente ou inválido
        """
        if not token or token.strip() == "":
            if required:
                raise ValueError(f"Token {token_name} é obrigatório mas não foi encontrado nas variáveis de ambiente")
            return False
            
        # Verificação básica de estrutura mínima usando configuração
        min_length = CONFIG["token_validation"]["min_length"]
        if len(token) < min_length or " " in token:
            if required:
                raise ValueError(f"Token {token_name} parece inválido (formato incorreto)")
            return False
            
        return True
    
    @staticmethod
    def validate_openai_token(token: Optional[str] = None, required: bool = True) -> bool:
        """
        Valida token da OpenAI, com verificações específicas para o formato do token.
        
        Args:
            token: Token da OpenAI. Se None, será buscado na variável de ambiente
            required: Se o token é obrigatório
            
        Returns:
            bool: True se o token for válido, False caso contrário
            
        Raises:
            ValueError: Se o token for obrigatório e estiver ausente ou inválido
        """
        # Se não fornecido, tenta buscar da variável de ambiente
        if token is None:
            token = os.environ.get("OPENAI_API_KEY", "")
            
        # Verificação específica para tokens da OpenAI usando configuração
        openai_prefix = CONFIG["token_validation"]["openai_prefix"]
        if token and not token.startswith(openai_prefix):
            if required:
                raise ValueError(f"Token da OpenAI inválido (deve começar com '{openai_prefix}')")
            return False
            
        return TokenValidator.validate_token(token, "OpenAI", required)
        
    @staticmethod
    def validate_github_token(token: Optional[str] = None, required: bool = True) -> bool:
        """
        Valida token do GitHub, com verificações específicas para o formato do token.
        
        Args:
            token: Token do GitHub. Se None, será buscado na variável de ambiente
            required: Se o token é obrigatório
            
        Returns:
            bool: True se o token for válido, False caso contrário
            
        Raises:
            ValueError: Se o token for obrigatório e estiver ausente ou inválido
        """
        # Se não fornecido, tenta buscar da variável de ambiente
        if token is None:
            token = os.environ.get("GITHUB_TOKEN", "")
            
        # GitHub tokens geralmente têm um formato específico, como começar com "ghp_"
        # Mas isso pode variar, então faremos uma verificação básica
        
        return TokenValidator.validate_token(token, "GitHub", required)
    
    @staticmethod
    def validate_all_required_tokens() -> bool:
        """
        Valida todos os tokens obrigatórios para o sistema.
        
        Returns:
            bool: True se todos os tokens obrigatórios estiverem válidos
            
        Raises:
            ValueError: Detalhando quais tokens estão faltando ou são inválidos
        """
        missing_tokens = []
        
        try:
            TokenValidator.validate_openai_token(required=True)
        except ValueError as e:
            missing_tokens.append(str(e))
            
        try:
            TokenValidator.validate_github_token(required=True)
        except ValueError as e:
            missing_tokens.append(str(e))
            
        if missing_tokens:
            raise ValueError(f"Tokens obrigatórios faltando ou inválidos: {', '.join(missing_tokens)}")
            
        return True 

class VersionAnalyzer:
    """Analisador de versões do projeto."""
    
    def __init__(self):
        """Inicializa o analisador de versões."""
        self.version_file = '.version.json'
        self.load_version_data()
        
    def load_version_data(self) -> None:
        """Carrega dados de versão do arquivo."""
        try:
            with open(self.version_file, 'r', encoding='utf-8') as f:
                self.version_data = json.load(f)
        except FileNotFoundError:
            self.version_data = {
                "current": "0.1.0",
                "manifest": {
                    "include": [
                        "LICENSE",
                        "README.md",
                        "src/configs/*.yaml"
                    ]
                },
                "history": {
                    "0.1.0": {
                        "date": datetime.now().strftime("%Y-%m-%d"),
                        "commit": "initial",
                        "increment_type": "minor",
                        "previous_version": "0.0.0"
                    }
                }
            }
            self.save_version_data()
    
    def save_version_data(self) -> None:
        """Salva dados de versão no arquivo."""
        with open(self.version_file, 'w', encoding='utf-8') as f:
            json.dump(self.version_data, f, indent=4)
            
    def get_current_version(self) -> str:
        """Retorna a versão atual."""
        return self.version_data['current']
    
    def increment_version(self, current: str, increment_type: str) -> str:
        """
        Incrementa a versão seguindo semver.
        
        Args:
            current: Versão atual
            increment_type: Tipo de incremento (major, minor, patch)
            
        Returns:
            str: Nova versão
        """
        major, minor, patch = map(int, current.split('.'))
        
        if increment_type == 'major':
            major += 1
            minor = 0
            patch = 0
        elif increment_type == 'minor':
            minor += 1
            patch = 0
        else:  # patch
            patch += 1
            
        return f"{major}.{minor}.{patch}"
    
    def smart_bump(self) -> None:
        """
        Incrementa a versão automaticamente baseado no último commit.
        """
        try:
            # Carrega configurações do kernel
            kernel_config = load_config('src/configs/kernel.yaml')
            
            # Obtém o último commit
            result = os.popen('git log -1 --pretty=%B').read().strip().lower()
            
            # Determina o tipo de incremento
            increment_type = 'patch'  # default
            
            for pattern_type, patterns in kernel_config['patterns'].items():
                if any(p.lower() in result for p in patterns):
                    if pattern_type == 'breaking':
                        increment_type = 'major'
                        break
                    elif pattern_type == 'feature':
                        increment_type = 'minor'
                        break
            
            # Incrementa a versão
            current = self.get_current_version()
            new_version = self.increment_version(current, increment_type)
            
            # Atualiza o histórico
            self.version_data['history'][new_version] = {
                "date": datetime.now().strftime("%Y-%m-%d"),
                "commit": result,
                "increment_type": increment_type,
                "previous_version": current
            }
            
            # Atualiza a versão atual
            self.version_data['current'] = new_version
            
            # Salva as alterações
            self.save_version_data()
            
            logger.info(f"SUCESSO - Versão incrementada: {current} -> {new_version}")
            
        except Exception as e:
            logger.error(f"FALHA - Erro ao incrementar versão: {str(e)}", exc_info=True)
            raise


"""
Gerenciador de modelos de IA com suporte a múltiplos provedores e fallback automático.
"""
from enum import Enum
from typing import Any, Dict, Optional, Tuple
import os
import json
import time
import yaml
from pathlib import Path

import google.generativeai as genai
from pydantic import BaseModel
from openai import OpenAI
from anthropic import Anthropic

from src.core.kernel import get_env_var
from src.core.logger import get_logger

logger = get_logger(__name__)

def load_config() -> Dict[str, Any]:
    """
    Carrega as configurações do model manager do arquivo YAML.
    
    Returns:
        Dict com as configurações
    """
    base_dir = Path(__file__).resolve().parent.parent.parent
    config_path = os.path.join(base_dir, 'src', 'configs', 'models.yaml')
    with open(config_path, 'r') as f:
        return yaml.safe_load(f)

# Carrega configurações
CONFIG = load_config()

class ModelProvider(str, Enum):
    """Provedores de modelos suportados."""
    OPENAI = "openai"
    OPENROUTER = "openrouter"
    GEMINI = "gemini"


class ModelConfig(BaseModel):
    """Configuração de um modelo."""
    provider: ModelProvider
    model_id: str
    api_key: str
    timeout: int = CONFIG['defaults']['timeout']
    max_retries: int = CONFIG['defaults']['max_retries']
    temperature: float = CONFIG['defaults']['temperature']
    max_tokens: Optional[int] = CONFIG['defaults']['max_tokens']


class ModelManager:
    """Gerenciador de modelos de IA."""

    def __init__(self, model_name: Optional[str] = None):
        """
        Inicializa o gerenciador com configurações do modelo.
        
        Args:
            model_name: Nome do modelo a ser usado (opcional)
        """
        env = CONFIG['env_vars']
        self.model_name = model_name or get_env_var(env['default_model'], CONFIG['defaults']['model'])
        self.elevation_model = get_env_var(env['elevation_model'], CONFIG['defaults']['elevation_model'])
        
        # Configurações de retry e timeout
        self.max_retries = int(get_env_var(env['max_retries'], str(CONFIG['defaults']['max_retries'])))
        self.timeout = int(get_env_var(env['model_timeout'], str(CONFIG['defaults']['timeout'])))
        
        # Configuração de fallback
        self.fallback_enabled = get_env_var(env['fallback_enabled'], str(CONFIG['fallback']['enabled'])).lower() == 'true'
        
        # Cache de respostas
        self.cache_enabled = get_env_var(env['cache_enabled'], str(CONFIG['cache']['enabled'])).lower() == 'true'
        self.cache_ttl = int(get_env_var(env['cache_ttl'], str(CONFIG['cache']['ttl'])))
        self.cache_dir = get_env_var(env['cache_dir'], CONFIG['cache']['directory'])
        self._setup_cache()
        
        # Inicializa clientes
        self._setup_clients()
        
        # Configurações padrão
        self.temperature = CONFIG['defaults']['temperature']
        self.max_tokens = CONFIG['defaults']['max_tokens']
        
        logger.info(f"ModelManager inicializado com modelo {self.model_name}")

    def configure(self, model: Optional[str] = None, temperature: float = CONFIG['defaults']['temperature'], max_tokens: Optional[int] = None) -> None:
        """
        Configura parâmetros do modelo.
        
        Args:
            model: Nome do modelo a ser usado
            temperature: Temperatura para geração (0.0 a 1.0)
            max_tokens: Número máximo de tokens na resposta
        """
        if model:
            self.model_name = model
        self.temperature = temperature
        self.max_tokens = max_tokens
        
        logger.info(f"ModelManager configurado: model={self.model_name}, temperature={self.temperature}, max_tokens={self.max_tokens}")

    def _setup_cache(self) -> None:
        """Configura diretório de cache"""
        if self.cache_enabled:
            os.makedirs(self.cache_dir, exist_ok=True)
            
    def _setup_clients(self) -> None:
        """Inicializa clientes para diferentes provedores"""
        env = CONFIG['env_vars']
        
        # OpenAI
        self.openai_client = OpenAI(
            api_key=get_env_var(env['openai_key']),
            timeout=self.timeout
        )
        
        # OpenRouter (opcional)
        openrouter_key = get_env_var(env['openrouter_key'])
        if openrouter_key:
            self.openrouter_client = OpenAI(
                base_url=CONFIG['providers']['openrouter']['base_url'],
                api_key=openrouter_key,
                timeout=self.timeout
            )
        else:
            self.openrouter_client = None
            
        # Gemini (opcional)
        gemini_key = get_env_var(env['gemini_key'])
        if gemini_key:
            genai.configure(api_key=gemini_key)
            self.gemini_model = genai.GenerativeModel(CONFIG['providers']['gemini']['default_model'])
        else:
            self.gemini_model = None
            
        # Anthropic (opcional)
        anthropic_key = get_env_var(env['anthropic_key'])
        if anthropic_key:
            self.anthropic_client = Anthropic(api_key=anthropic_key)
        else:
            self.anthropic_client = None
            
    def _get_cache_key(self, prompt: str, system: Optional[str] = None, **kwargs) -> str:
        """
        Gera chave de cache para um prompt.
        
        Args:
            prompt: Prompt para o modelo
            system: Prompt de sistema (opcional)
            **kwargs: Argumentos adicionais
            
        Returns:
            String com a chave de cache
        """
        cache_key = {
            "prompt": prompt,
            "system": system,
            "model": self.model_name,
            "temperature": self.temperature,
            "max_tokens": self.max_tokens,
            **kwargs
        }
        return json.dumps(cache_key, sort_keys=True)
        
    def _get_cached_response(self, cache_key: str) -> Optional[Tuple[str, Dict[str, Any]]]:
        """
        Obtém resposta do cache se disponível.
        
        Args:
            cache_key: Chave de cache
            
        Returns:
            Tupla (resposta, metadados) se encontrada, None caso contrário
        """
        if not self.cache_enabled:
            return None
            
        cache_file = os.path.join(self.cache_dir, f"{cache_key}.json")
        if not os.path.exists(cache_file):
            return None
            
        try:
            with open(cache_file, 'r') as f:
                cache_data = json.load(f)
                
            # Verifica TTL
            if time.time() - cache_data['timestamp'] > self.cache_ttl:
                os.remove(cache_file)
                return None
                
            return cache_data['response'], cache_data['metadata']
            
        except Exception as e:
            logger.error(f"Erro ao ler cache: {str(e)}")
            return None
            
    def _save_to_cache(self, cache_key: str, response: str, metadata: Dict[str, Any]) -> None:
        """
        Salva resposta no cache.
        
        Args:
            cache_key: Chave de cache
            response: Resposta do modelo
            metadata: Metadados da resposta
        """
        if not self.cache_enabled:
            return
            
        try:
            cache_data = {
                'response': response,
                'metadata': metadata,
                'timestamp': time.time()
            }
            
            cache_file = os.path.join(self.cache_dir, f"{cache_key}.json")
            with open(cache_file, 'w') as f:
                json.dump(cache_data, f)
                
        except Exception as e:
            logger.error(f"Erro ao salvar cache: {str(e)}")

    def _get_provider(self, model: str) -> str:
        """
        Identifica o provedor com base no nome do modelo.
        
        Args:
            model: Nome do modelo
            
        Returns:
            String com o nome do provedor
        """
        providers_config = CONFIG['providers']
        
        # Verifica cada provedor
        for provider, config in providers_config.items():
            if 'prefix_patterns' in config:
                for prefix in config['prefix_patterns']:
                    if model.startswith(prefix):
                        return provider
                        
        # Retorna OpenAI como default
        return 'openai'

    def _get_provider_config(self, provider: str) -> Dict[str, Any]:
        """
        Obtém configurações específicas do provedor.
        
        Args:
            provider: Nome do provedor
            
        Returns:
            Dict com configurações do provedor
        """
        return CONFIG['providers'].get(provider, {})

    def _generate_with_provider(
        self,
        provider: str,
        prompt: str,
        system: Optional[str] = None,
        **kwargs
    ) -> Tuple[str, Dict[str, Any]]:
        """
        Gera resposta usando um provedor específico.
        
        Args:
            provider: Nome do provedor
            prompt: Prompt para o modelo
            system: Prompt de sistema (opcional)
            **kwargs: Argumentos adicionais
            
        Returns:
            Tupla (resposta, metadados)
        """
        provider_config = self._get_provider_config(provider)
        
        try:
            if provider == 'openai':
                messages = []
                if system:
                    messages.append({"role": "system", "content": system})
                messages.append({"role": "user", "content": prompt})
                
                response = self.openai_client.chat.completions.create(
                    model=self.model_name,
                    messages=messages,
                    temperature=self.temperature,
                    max_tokens=self.max_tokens,
                    **kwargs
                )
                return response.choices[0].message.content, {
                    "model": self.model_name,
                    "provider": provider,
                    "status": "success"
                }
                
            elif provider == 'openrouter' and self.openrouter_client:
                messages = []
                if system:
                    messages.append({"role": "system", "content": system})
                messages.append({"role": "user", "content": prompt})
                
                response = self.openrouter_client.chat.completions.create(
                    model=self.model_name,
                    messages=messages,
                    temperature=self.temperature,
                    max_tokens=self.max_tokens,
                    **kwargs
                )
                return response.choices[0].message.content, {
                    "model": self.model_name,
                    "provider": provider,
                    "status": "success"
                }
                
            elif provider == 'gemini' and self.gemini_model:
                response = self.gemini_model.generate_content(
                    prompt,
                    generation_config=genai.types.GenerationConfig(
                        temperature=self.temperature,
                        max_output_tokens=self.max_tokens or provider_config.get('default_max_tokens', 1024),
                        **kwargs
                    )
                )
                return response.text, {
                    "model": provider_config['default_model'],
                    "provider": provider,
                    "status": "success"
                }
                
            elif provider == 'anthropic' and self.anthropic_client:
                messages = []
                if system:
                    messages.append({"role": "system", "content": system})
                messages.append({"role": "user", "content": prompt})
                
                response = self.anthropic_client.messages.create(
                    model=self.model_name,
                    messages=messages,
                    temperature=self.temperature,
                    max_tokens=self.max_tokens or provider_config.get('default_max_tokens', 1024),
                    **kwargs
                )
                return response.content[0].text, {
                    "model": self.model_name,
                    "provider": provider,
                    "status": "success"
                }
                
            else:
                raise ValueError(f"Provedor {provider} não disponível")
                
        except Exception as e:
            logger.error(f"Erro ao gerar com provedor {provider}: {str(e)}")
            return "", {
                "error": str(e),
                "model": self.model_name,
                "provider": provider,
                "status": "error"
            }

    def generate(
        self,
        prompt: str,
        system: Optional[str] = None,
        use_cache: bool = True,
        **kwargs
    ) -> Tuple[str, Dict[str, Any]]:
        """
        Gera resposta para um prompt.
        
        Args:
            prompt: Prompt para o modelo
            system: Prompt de sistema (opcional)
            use_cache: Se deve usar cache
            **kwargs: Argumentos adicionais
            
        Returns:
            Tupla (resposta, metadados)
        """
        # Verifica cache
        if use_cache and self.cache_enabled:
            cache_key = self._get_cache_key(prompt, system, **kwargs)
            cached = self._get_cached_response(cache_key)
            if cached:
                return cached

        # Identifica provedor
        provider = self._get_provider(self.model_name)
        
        # Tenta gerar resposta
        for attempt in range(self.max_retries):
            try:
                response, metadata = self._generate_with_provider(
                    provider,
                    prompt,
                    system,
                    **kwargs
                )
                
                if metadata['status'] == 'success':
                    # Salva no cache
                    if use_cache and self.cache_enabled:
                        self._save_to_cache(cache_key, response, metadata)
                    return response, metadata
                    
                # Se falhou e fallback está habilitado, tenta outro provedor
                if self.fallback_enabled and attempt == self.max_retries - 1:
                    logger.warning(f"Fallback para modelo {self.elevation_model}")
                    self.model_name = self.elevation_model
                    provider = self._get_provider(self.model_name)
                    continue
                    
            except Exception as e:
                logger.error(f"Tentativa {attempt + 1} falhou: {str(e)}")
                if attempt == self.max_retries - 1:
                    return "", {
                        "error": str(e),
                        "model": self.model_name,
                        "provider": provider,
                        "status": "error"
                    }
                    
        return "", {
            "error": "Máximo de tentativas excedido",
            "model": self.model_name,
            "provider": provider,
            "status": "error"
        } 


"""
CLI para o sistema.
Módulo principal que define os comandos disponíveis na interface de linha de comando.
"""
import json
import sys
import time
from typing import Dict

import typer
from rich.console import Console
from rich.markdown import Markdown

from src.app import AgentOrchestrator
from src.core.models import ModelManager
from src.core.logger import get_logger, log_execution
from src.core.kernel import get_env_var, get_env_status as get_kernel_env_status, validate_env as validate_kernel_env

logger = get_logger(__name__)

app = typer.Typer()
output_console = Console()  # Console para saída normal

def get_env_status() -> Dict[str, Dict[str, bool]]:
    """
    Obtém o status das variáveis de ambiente.

    Returns:
        Dict com o status das variáveis.
    """
    return get_kernel_env_status("cli")

@log_execution
def validate_env() -> None:
    """
    Valida se todas as variáveis de ambiente obrigatórias estão definidas.

    Raises:
        ValueError: Se alguma variável obrigatória não estiver definida.
    """
    try:
        validate_kernel_env("cli")
    except ValueError as e:
        logger.error(str(e))
        print(str(e), file=sys.stderr)
        raise typer.Exit(code=1)

def get_orchestrator() -> AgentOrchestrator:
    """
    Retorna uma instância do AgentOrchestrator.
    
    Returns:
        AgentOrchestrator: Nova instância do orquestrador
    """
    return AgentOrchestrator()

@app.command()
def status():
    """
    Exibe o status do ambiente e configurações.
    """
    try:
        # Obtém status do ambiente
        env_status = get_env_status()
        
        # Obtém modelos disponíveis
        models = ModelManager()
        available_models = models.get_available_models()
        
        # Formata saída
        status = {
            "environment": env_status,
            "models": available_models
        }
        
        output_console.print(json.dumps(status, indent=2))
        return 0
        
    except Exception as e:
        error_msg = f"Erro ao processar comando: {str(e)}"
        logger.error(error_msg, exc_info=True)
        print(error_msg, file=sys.stderr)
        raise typer.Exit(code=1)

@app.command()
def mcp(
    prompt_tdd: str = typer.Argument(""),
    format: str = typer.Option("json", help="Formato de saída (json, markdown)"),
    model: str = typer.Option("gpt-3.5-turbo", help="Modelo a ser usado"),
    temperature: float = typer.Option(0.7, help="Temperatura para geração")
):
    """
    Executa o Agent Flow TDD em modo MCP.
    """
    try:
        print("🛠️ Executando CLI em modo desenvolvimento...")
        
        # Valida ambiente
        validate_env()
        
        from src.mcp import MCPHandler
        
        # Inicializa handler MCP
        handler = MCPHandler()
        handler.initialize(api_key=get_env_var("OPENAI_API_KEY"))
        
        # Executa loop MCP
        handler.run()
        return 0
        
    except Exception as e:
        error_msg = f"Erro ao processar comando: {str(e)}"
        logger.error(error_msg, exc_info=True)
        print(error_msg, file=sys.stderr)
        raise typer.Exit(code=1)

@app.command()
def feature(
    prompt_tdd: str = typer.Argument(..., help="Prompt para o TDD"),
    format: str = typer.Option("json", help="Formato de saída (json, markdown)"),
    model: str = typer.Option("gpt-3.5-turbo", help="Modelo a ser usado"),
    temperature: float = typer.Option(0.7, help="Temperatura para geração")
):
    """
    Executa o Agent Flow TDD para gerar uma feature.
    """
    try:
        print("🛠️ Executando CLI em modo desenvolvimento...")
        
        # Valida ambiente
        validate_env()
        
        # Inicializa orquestrador
        orchestrator = get_orchestrator()
        
        # Executa o prompt
        result = orchestrator.execute(
            prompt=prompt_tdd,
            model=model,
            temperature=temperature,
            session_id=str(time.time()),
            format=format
        )
        
        # Formata e exibe resultado
        if format == "markdown":
            output_console.print(Markdown(result.output))
        else:
            try:
                # Tenta carregar como JSON
                content = json.loads(result.output)
            except json.JSONDecodeError:
                # Se falhar, usa como string
                content = result.output
                
            output = {
                "content": content,
                "metadata": {
                    "type": "feature",
                    "options": {
                        "format": format,
                        "model": model,
                        "temperature": temperature
                    }
                }
            }
            print(json.dumps(output, ensure_ascii=False))
            
        return 0
        
    except Exception as e:
        error_msg = f"Erro ao processar comando: {str(e)}"
        logger.error(error_msg, exc_info=True)
        print(error_msg, file=sys.stderr)
        raise typer.Exit(code=1)

if __name__ == "__main__":
    app()



# src/core/logger.py
import os
import logging
import logging.handlers
import uuid
import time
import re
from contextvars import ContextVar
from pathlib import Path
from functools import wraps
from typing import Optional, Dict, Any, List, Tuple, Callable
from dataclasses import dataclass, field
import json
from datetime import datetime
import functools
import yaml

# Diretório base do projeto
BASE_DIR = Path(__file__).resolve().parent.parent.parent

def load_config() -> Dict[str, Any]:
    """
    Carrega as configurações do logger do arquivo YAML.
    
    Returns:
        Dict com as configurações
    """
    config_path = os.path.join(BASE_DIR, 'src', 'configs', 'logger.yaml')
    with open(config_path, 'r') as f:
        return yaml.safe_load(f)

# Carrega configurações
CONFIG = load_config()

# Configura diretório de logs
LOG_DIR = os.path.join(BASE_DIR, CONFIG['directories']['logs'])
os.makedirs(LOG_DIR, exist_ok=True)

# Configuração de níveis de log
LOG_LEVEL_MAP = CONFIG['log_levels']['map']
DEFAULT_LOG_LEVEL = CONFIG['log_levels']['default']
LOG_LEVEL = os.environ.get('LOG_LEVEL', DEFAULT_LOG_LEVEL).upper()
NUMERIC_LOG_LEVEL = LOG_LEVEL_MAP.get(LOG_LEVEL, LOG_LEVEL_MAP['INFO'])

# Lista de palavras-chave para identificar dados sensíveis
SENSITIVE_KEYWORDS = CONFIG['security']['sensitive_keywords']

# Padrões de tokens a serem mascarados
TOKEN_PATTERNS = CONFIG['security']['token_patterns']

class SecureLogFilter(logging.Filter):
    """Filtro para mascarar dados sensíveis nos registros de log"""
    
    def __init__(self):
        super().__init__()
        self.mask_config = CONFIG['security']['masking']
    
    def filter(self, record: logging.LogRecord) -> bool:
        """Processa e mascara dados sensíveis no registro de log"""
        record.msg = self.mask_sensitive_data(record.msg)
        if isinstance(record.args, dict):
            record.args = self.mask_sensitive_data(record.args)
        else:
            record.args = tuple(self.mask_sensitive_data(arg) for arg in record.args)
        return True

    def mask_sensitive_data(self, data: Any, mask_str: Optional[str] = None) -> Any:
        """Mascara dados sensíveis em strings e estruturas de dados"""
        mask_str = mask_str or self.mask_config['default_mask']
        if isinstance(data, dict):
            return {k: self.mask_value(k, v, mask_str) for k, v in data.items()}
        elif isinstance(data, list):
            return [self.mask_sensitive_data(item, mask_str) for item in data]
        elif isinstance(data, str):
            return self.mask_string(data, mask_str)
        return data

    def mask_value(self, key: str, value: Any, mask_str: str) -> Any:
        """Mascara valores sensíveis baseado na chave e conteúdo"""
        if any(keyword in key.lower() for keyword in SENSITIVE_KEYWORDS):
            return mask_str
        return self.mask_sensitive_data(value, mask_str)

    def mask_string(self, text: str, mask_str: str) -> str:
        """Mascara padrões sensíveis em strings"""
        if len(text) > self.mask_config['min_length_for_masking']:
            for pattern in TOKEN_PATTERNS:
                if re.search(pattern, text):
                    return self.mask_partially(text, mask_str)
        if any(keyword in text.lower() for keyword in SENSITIVE_KEYWORDS):
            return self.mask_partially(text, mask_str)
        return text

    def mask_partially(self, text: str, mask_str: str) -> str:
        """Mascara parcialmente mantendo parte do conteúdo"""
        if len(text) <= self.mask_config['min_length_for_partial']:
            return mask_str
        prefix = text[:self.mask_config['prefix_length']]
        suffix = text[-self.mask_config['suffix_length']:] if len(text) > (self.mask_config['prefix_length'] + self.mask_config['suffix_length']) else ''
        return f"{prefix}{mask_str}{suffix}"

def setup_logger(name: str, level: Optional[str] = None) -> logging.Logger:
    """
    Configura um logger com o formato padrão.
    
    Args:
        name: Nome do logger
        level: Nível de log (DEBUG, INFO, etc)
        
    Returns:
        Logger configurado
    """
    # Cria logger
    logger = logging.getLogger(name)
    
    # Define nível
    log_level = level or os.getenv("LOG_LEVEL", CONFIG['log_levels']['default'])
    logger.setLevel(getattr(logging, log_level))
    
    # Remove handlers existentes
    for handler in logger.handlers[:]:
        logger.removeHandler(handler)
    
    # Adiciona handler de console
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG)
    
    # Define formato
    formatter = logging.Formatter(CONFIG['format']['default'])
    console_handler.setFormatter(formatter)
    
    # Adiciona filtro de segurança
    secure_filter = SecureLogFilter()
    console_handler.addFilter(secure_filter)
    
    # Adiciona handler
    logger.addHandler(console_handler)
    
    return logger

def log_execution(func: Callable) -> Callable:
    """
    Decorador para logging de execução de funções.
    
    Args:
        func: Função a ser decorada
        
    Returns:
        Função decorada com logging
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        logger = get_logger(func.__module__)
        
        # Log de início
        logger.info(f"INÍCIO - {func.__name__} | Args: {args}, Kwargs: {kwargs}")
        
        try:
            # Executa função
            result = func(*args, **kwargs)
            
            # Log de sucesso
            logger.info(f"SUCESSO - {func.__name__}")
            return result
            
        except Exception as e:
            # Log de erro
            logger.error(
                f"FALHA - {func.__name__} | Erro: {str(e)}", 
                exc_info=True
            )
            raise
            
    return wrapper

def log_error(error: Exception, context: Optional[Dict[str, Any]] = None) -> None:
    """
    Loga um erro com contexto adicional.
    
    Args:
        error: Exceção a ser logada
        context: Dicionário com informações de contexto
    """
    logger = get_logger(__name__)
    
    error_data = {
        'error_type': type(error).__name__,
        'error_message': str(error),
        'timestamp': datetime.now().isoformat(),
        'context': context or {}
    }
    
    logger.error(
        f"Erro: {json.dumps(error_data, indent=2)}", 
        exc_info=True
    )

# Alias para compatibilidade
get_logger = setup_logger

# Logger global padrão
logger = setup_logger('agent_flow_tdd')

# Funções auxiliares de conveniência
def log_error(message: str, exc_info=False) -> None:
    """Registra erro com stacktrace opcional"""
    logger.error(message, exc_info=exc_info)

def log_warning(message: str) -> None:
    """Registra aviso"""
    logger.warning(message)

def log_info(message: str) -> None:
    """Registra informação"""
    logger.info(message)

def log_debug(message: str) -> None:
    """Registra mensagem de debug"""
    logger.debug(message)

def get_child_logger(name: str) -> logging.Logger:
    """Obtém um logger filho configurado"""
    return logger.getChild(name)

# Context variables para tracing
current_trace: ContextVar[Optional['Trace']] = ContextVar('current_trace', default=None)
current_span: ContextVar[Optional['Span']] = ContextVar('current_span', default=None)

@dataclass
class Span:
    """Representa uma operação temporal dentro de um trace"""
    trace_id: str
    span_id: str = field(default_factory=lambda: f"{CONFIG['trace']['prefixes']['span']}{uuid.uuid4().hex}")
    parent_id: Optional[str] = None
    span_type: str = CONFIG['trace']['default_span_type']
    name: Optional[str] = None
    start_time: float = field(default_factory=time.time)
    end_time: Optional[float] = None
    span_data: Dict[str, Any] = field(default_factory=dict)
    error: Optional[str] = None

@dataclass
class Trace:
    """Representa um fluxo completo de execução"""
    trace_id: str = field(default_factory=lambda: f"{CONFIG['trace']['prefixes']['trace']}{uuid.uuid4().hex}")
    workflow_name: str = CONFIG['trace']['default_workflow_name']
    group_id: Optional[str] = None
    start_time: float = field(default_factory=time.time)
    end_time: Optional[float] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    spans: List[Span] = field(default_factory=list)
    disabled: bool = False
    config: 'TraceConfig' = field(default_factory=lambda: TraceConfig())

@dataclass
class TraceConfig:
    """Configuração para controle do tracing"""
    tracing_disabled: bool = CONFIG['trace']['tracing_disabled']
    trace_include_sensitive_data: bool = CONFIG['trace']['trace_include_sensitive_data']
    trace_processors: List['TraceProcessor'] = field(default_factory=list)

class TraceProcessor:
    """Interface para processamento de traces"""
    def process_trace(self, trace: Trace):
        raise NotImplementedError

class FileTraceProcessor(TraceProcessor):
    """Processador que salva traces em arquivo"""
    
    def __init__(self, file_path: Optional[str] = None):
        self.file_path = file_path or CONFIG['trace']['file_processor']['default_file']

def trace(
    workflow_name: str = CONFIG['trace']['default_workflow_name'],
    group_id: Optional[str] = None,
    disabled: Optional[bool] = None,
    metadata: Optional[Dict] = None
):
    """Context manager para criação de traces"""
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            logger = logging.getLogger('agent_flow_tdd')
            if logger.trace_config.tracing_disabled or disabled:
                return func(*args, **kwargs)

            current_trace.get()
            new_trace = Trace(
                workflow_name=workflow_name,
                group_id=group_id,
                metadata=metadata or {},
                config=logger.trace_config
            )

            token = current_trace.set(new_trace)
            try:
                result = func(*args, **kwargs)
                new_trace.end_time = time.time()
                
                # Processar trace
                for processor in logger.trace_config.trace_processors:
                    processor.process_trace(new_trace)
                
                return result
            except Exception as e:
                new_trace.end_time = time.time()
                new_trace.metadata['error'] = str(e)
                raise
            finally:
                current_trace.reset(token)

        return wrapper
    return decorator

def span(
    span_type: str = CONFIG['trace']['default_span_type'],
    name: Optional[str] = None,
    capture_args: bool = False
):
    """Decorador para criação de spans"""
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            current_t = current_trace.get()
            current_s = current_span.get()
            
            if not current_t or current_t.disabled:
                return func(*args, **kwargs)

            new_span = Span(
                trace_id=current_t.trace_id,
                parent_id=current_s.span_id if current_s else None,
                span_type=span_type,
                name=name or func.__name__,
                span_data={
                    'function': func.__name__,
                    'module': func.__module__,
                    'args': mask_arguments(args, kwargs) if capture_args else None
                }
            )

            token = current_span.set(new_span)
            try:
                result = func(*args, **kwargs)
                new_span.end_time = time.time()
                current_t.spans.append(new_span)
                return result
            except Exception as e:
                new_span.end_time = time.time()
                new_span.error = str(e)
                current_t.spans.append(new_span)
                raise
            finally:
                current_span.reset(token)

        return wrapper
    return decorator

def mask_arguments(args: Tuple, kwargs: Dict) -> Dict:
    """Mascara argumentos sensíveis para inclusão nos spans"""
    masked_args = [SecureLogFilter().mask_sensitive_data(arg) for arg in args]
    masked_kwargs = {
        k: SecureLogFilter().mask_sensitive_data(v) if 'password' not in k else '***'
        for k, v in kwargs.items()
    }
    return {'args': masked_args, 'kwargs': masked_kwargs}

# Span types específicos
def agent_span(name: str = "Agent Run"):
    return span(span_type="agent", name=name, capture_args=True)

def generation_span(name: str = "LLM Generation"):
    return span(span_type="generation", name=name)

def tool_span(name: str = "Tool Execution"):
    return span(span_type="tool", name=name, capture_args=True)



"""
Módulo de gerenciamento do banco de dados.
"""
import json
import logging
import os
import sqlite3
from typing import Any, Dict, List, Optional
import yaml

logger = logging.getLogger(__name__)

class DatabaseManager:
    """Gerenciador do banco de dados SQLite."""
    
    def __init__(self, db_path: Optional[str] = None):
        """
        Inicializa o gerenciador.
        
        Args:
            db_path: Caminho para o banco de dados
        """
        # Carrega configurações
        config_path = os.path.join("src", "configs", "database.yaml")
        with open(config_path, "r") as f:
            self.config = yaml.safe_load(f)
        
        # Cria diretório de logs se não existir
        os.makedirs(self.config["directories"]["logs"], exist_ok=True)
        
        # Define caminho do banco
        self.db_path = db_path or self.config["database"]["default_path"]
        
        # Conecta ao banco
        self.conn = sqlite3.connect(self.db_path)
        self.conn.row_factory = sqlite3.Row
        
        # Cria tabelas
        self._create_tables()
        
    def _create_tables(self):
        """Cria as tabelas do banco de dados."""
        cursor = self.conn.cursor()
        
        for table_name, table_config in self.config["tables"].items():
            # Monta SQL para colunas
            columns = []
            for col in table_config["columns"]:
                columns.append(f"{col['name']} {col['type']}")
            
            # Adiciona foreign keys
            if "foreign_keys" in table_config:
                for fk in table_config["foreign_keys"]:
                    columns.append(f"FOREIGN KEY({fk['column']}) REFERENCES {fk['references']}")
            
            # Adiciona checks
            if "checks" in table_config:
                for check in table_config["checks"]:
                    columns.append(f"CHECK({check['column']} {check['constraint']})")
            
            # Monta e executa SQL final
            sql = f"""
            CREATE TABLE IF NOT EXISTS {table_config['name']} (
                {', '.join(columns)}
            )
            """
            cursor.execute(sql)
        
        self.conn.commit()
    
    def log_run(self, session_id: str, input: str, final_output: Optional[str] = None,
                last_agent: Optional[str] = None, output_type: Optional[str] = None) -> int:
        """
        Registra uma execução do agente.
        
        Args:
            session_id: ID da sessão
            input: Texto de entrada
            final_output: Saída final
            last_agent: Último agente executado
            output_type: Tipo de saída
            
        Returns:
            ID da execução registrada
        """
        cursor = self.conn.cursor()
        
        cursor.execute("""
        INSERT INTO agent_runs (session_id, input, final_output, last_agent, output_type)
        VALUES (?, ?, ?, ?, ?)
        """, (session_id, input, final_output, last_agent, output_type))
        
        self.conn.commit()
        return cursor.lastrowid
    
    def log_run_item(self, run_id: int, item_type: str, raw_item: Dict[str, Any],
                     source_agent: Optional[str] = None, target_agent: Optional[str] = None):
        """
        Registra um item gerado durante a execução.
        
        Args:
            run_id: ID da execução
            item_type: Tipo do item
            raw_item: Item bruto
            source_agent: Agente de origem
            target_agent: Agente de destino
        """
        cursor = self.conn.cursor()
        
        cursor.execute("""
        INSERT INTO run_items (run_id, item_type, raw_item, source_agent, target_agent)
        VALUES (?, ?, ?, ?, ?)
        """, (run_id, item_type, json.dumps(raw_item), source_agent, target_agent))
        
        self.conn.commit()
    
    def log_guardrail_results(self, run_id: int, guardrail_type: str, results: Dict[str, Any]):
        """
        Registra resultados de guardrails.
        
        Args:
            run_id: ID da execução
            guardrail_type: Tipo do guardrail (input/output)
            results: Resultados do guardrail
        """
        cursor = self.conn.cursor()
        
        cursor.execute("""
        INSERT INTO guardrail_results (run_id, guardrail_type, results)
        VALUES (?, ?, ?)
        """, (run_id, guardrail_type, json.dumps(results)))
        
        self.conn.commit()
    
    def log_raw_response(self, run_id: int, response: Dict[str, Any]):
        """
        Registra uma resposta bruta do LLM.
        
        Args:
            run_id: ID da execução
            response: Resposta do LLM
        """
        cursor = self.conn.cursor()
        
        cursor.execute("""
        INSERT INTO raw_responses (run_id, response)
        VALUES (?, ?)
        """, (run_id, json.dumps(response)))
        
        self.conn.commit()
    
    def get_run_history(self, limit: Optional[int] = None) -> List[Dict[str, Any]]:
        """
        Retorna o histórico das últimas execuções.
        
        Args:
            limit: Número máximo de registros. Se None, usa o valor padrão da configuração.
            
        Returns:
            Lista de execuções com seus detalhes, ordenada por timestamp decrescente
        """
        cursor = self.conn.cursor()
        
        # Usa limite da configuração se não especificado
        if limit is None:
            limit = self.config["database"]["history_limit"]
        
        # Busca execuções
        cursor.execute("""
        SELECT * FROM agent_runs
        ORDER BY id DESC
        LIMIT ?
        """, (limit,))
        
        runs = []
        for run in cursor.fetchall():
            run_dict = dict(run)
            
            # Busca itens
            cursor.execute("SELECT * FROM run_items WHERE run_id = ? ORDER BY id DESC", (run['id'],))
            run_dict['items'] = [dict(item) for item in cursor.fetchall()]
            
            # Busca guardrails
            cursor.execute("SELECT * FROM guardrail_results WHERE run_id = ? ORDER BY id DESC", (run['id'],))
            run_dict['guardrails'] = [dict(guardrail) for guardrail in cursor.fetchall()]
            
            # Busca respostas brutas
            cursor.execute("SELECT * FROM raw_responses WHERE run_id = ? ORDER BY id DESC", (run['id'],))
            run_dict['raw_responses'] = [dict(response) for response in cursor.fetchall()]
            
            runs.append(run_dict)
        
        return runs
    
    def close(self):
        """Fecha a conexão com o banco."""
        self.conn.close()


#!/usr/bin/env python3
"""
Script de instalação do pacote.
"""
import json
import subprocess
import sys
from setuptools import find_packages, setup
from setuptools.command.develop import develop
from setuptools.command.install import install
import pkg_resources

class PreInstallCommand:
    """Classe base para executar comandos antes da instalação."""
    def check_if_installed(self):
        """Verifica se o pacote já está instalado."""
        try:
            pkg_resources.get_distribution('agent-flow-tdd')
            return True
        except pkg_resources.DistributionNotFound:
            return False

    def run_pre_install(self):
        """Executa a limpeza de instalações anteriores."""
        if not self.check_if_installed():
            print("ℹ️  Nenhuma instalação anterior encontrada. Prosseguindo com instalação...")
            return

        print("🧹 Removendo instalação anterior do pacote...")
        try:
            subprocess.check_call([sys.executable, "-m", "pip", "uninstall", "-y", "agent-flow-tdd"])
            print("✅ Pacote removido com sucesso!")
        except subprocess.CalledProcessError as e:
            print(f"⚠️  Erro ao remover pacote: {e}")

class CustomInstallCommand(install, PreInstallCommand):
    """Comando customizado para instalação normal."""
    def run(self):
        self.run_pre_install()
        install.run(self)

class CustomDevelopCommand(develop, PreInstallCommand):
    """Comando customizado para instalação em modo desenvolvimento."""
    def run(self):
        self.run_pre_install()
        develop.run(self)

# Dependências principais
install_requires = [
    "typer>=0.9.0",
    "rich>=13.7.0",
    "openai>=1.12.0",
    "openrouter>=1.0.0",
    "google-generativeai>=0.8.0",
    "python-dotenv>=1.0.0",
    "pydantic>=2.6.0",
    "requests>=2.31.0",
    "tenacity>=8.2.0",
    "cachetools>=5.3.0",
    "anthropic>=0.18.0",
    "PyYAML>=6.0.1",
]

# Dependências de desenvolvimento
dev_requires = [
    "pytest>=8.0.0",
    "pytest-cov>=4.1.0",
    "pytest-mock>=3.12.0",
    "black>=23.0.0",
    "isort>=5.12.0",
    "flake8>=6.1.0",
    "mypy>=1.5.0",
    "autoflake>=2.2.0",
    "build>=1.0.3",
    "twine>=4.0.2",
]

# Lê a versão do arquivo .version.json
with open('.version.json', 'r', encoding='utf-8') as f:
    version_data = json.load(f)
    version = version_data['current']

setup(
    name="agent-flow-tdd",
    version=version,
    packages=find_packages(),
    package_data={
        '': ['.version.json'],  # Inclui arquivo de versão na raiz
        'src': ['configs/*.yaml'],  # Inclui arquivos YAML do diretório configs
    },
    data_files=[
        ('', ['.version.json']),  # Copia arquivo de versão para a raiz do pacote instalado
    ],
    install_requires=install_requires,
    extras_require={
        "dev": dev_requires
    },
    entry_points={
        "console_scripts": [
            "agent-flow-tdd=src.cli:app"
        ]
    },
    python_requires=">=3.8",
    author="Seu Nome",
    author_email="seu.email@exemplo.com",
    description="Framework para desenvolvimento orientado a testes com agentes de IA",
    long_description=open("README.md", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/seu-usuario/agent-flow-tdd",
    cmdclass={
        'install': CustomInstallCommand,
        'develop': CustomDevelopCommand,
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
) 


"""
Orquestrador de agentes do sistema.
"""
from typing import Any, Dict, List
from pydantic import BaseModel

from src.core import ModelManager
from src.core.db import DatabaseManager
from src.core.logger import get_logger

logger = get_logger(__name__)


class AgentResult(BaseModel):
    """Resultado de uma execução do agente."""
    output: Any
    items: List[Dict[str, Any]] = []
    guardrails: List[Dict[str, Any]] = []
    raw_responses: List[Dict[str, Any]] = []


class AgentOrchestrator:
    """Orquestrador de agentes do sistema."""

    def __init__(self):
        """Inicializa o orquestrador."""
        self.models = ModelManager()
        self.db = DatabaseManager()
        logger.info("AgentOrchestrator inicializado")

    def execute(self, prompt: str, **kwargs) -> AgentResult:
        """
        Executa o processamento do prompt.
        
        Args:
            prompt: Texto de entrada
            **kwargs: Argumentos adicionais
            
        Returns:
            AgentResult com o resultado do processamento
        """
        try:
            logger.info(f"INÍCIO - execute | Prompt: {prompt[:100]}...")
            
            # Configura o modelo
            self.models.configure(
                model=kwargs.get("model", "gpt-3.5-turbo"),
                temperature=kwargs.get("temperature", 0.7)
            )
            
            # Gera resposta
            text, metadata = self.models.generate(prompt)
            
            # Processa o resultado
            result = AgentResult(
                output=text,
                items=[],  # Implementar geração de itens
                guardrails=[],  # Implementar verificação de guardrails
                raw_responses=[{
                    "id": metadata.get("id"),
                    "response": metadata
                }]
            )
            
            # Registra no banco de dados
            run_id = self.db.log_run(
                session_id=kwargs.get("session_id", "default"),
                input=prompt,
                final_output=result.output,
                last_agent="OpenAI",
                output_type=kwargs.get("format", "json")  # Usa o formato passado nos kwargs
            )
            
            # Registra itens gerados
            for item in result.items:
                self.db.log_run_item(run_id, "item", item)
                
            # Registra guardrails
            for guardrail in result.guardrails:
                self.db.log_guardrail_results(run_id, "output", guardrail)
                
            # Registra respostas brutas
            for response in result.raw_responses:
                self.db.log_raw_response(run_id, response)
            
            logger.info(f"SUCESSO - execute | Tamanho da resposta: {len(result.output)}")
            return result
            
        except Exception as e:
            logger.error(f"FALHA - execute | Erro: {str(e)}", exc_info=True)
            raise
        finally:
            self.db.close()

# Uso
if __name__ == "__main__":
    orchestrator = AgentOrchestrator()
    user_prompt = "Preciso de um sistema de login com autenticação de dois fatores"
    result = orchestrator.execute(user_prompt)
    print("Resultado Final:", result.output)
